<?php

	// This file contains the code for crawling a page to search for authorship markup,
	// and to crawl author pages as needed. It also contains the global array that
	// is used to cache details from author pages, so they are only crawled once.
	
	$author_ids_for_author_page = array(); // author page cache
	
	// Crawls the page checking for authorship markup with a link to Google+ profile,
	// it will crawl subsequent author pages as necessary (which are also cached, should
	// a subsequent URL also link to the same author page).
	//
	// Returns a Google+ ID
	
	function get_page_author_id($page_url, $is_author_page = false)
	{
		global $author_ids_for_author_page;
		
		echo "Examining URL $page_url\n";
		if ($is_author_page)
		{
			echo "	Is an author page.\n";
			if (array_key_exists($page_url, $author_ids_for_author_page))
			{
				echo "	Retrieving from cache.\n";
				return $author_ids_for_author_page[$page_url];
			}
		}
		// Increase the maximum execution time before we run CURL	
		set_time_limit(120);
	
		$ch = curl_init();
		
		curl_setopt($ch, CURLOPT_URL, $page_url); 
		curl_setopt($ch, CURLOPT_TIMEOUT, 180);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		
		$page_source = curl_exec($ch);
		//$info = curl_getinfo($ch);
		curl_close($ch);
		
		$page_source = str_replace("\r", "", $page_source);
		$page_source = str_replace("\n", "", $page_source);
		
		// General process:
		// find author id
		// is it Google+ - if so return it
		// if not, then is it same domain
		// if not, return -1, if so then
		// call this function on it
		// cache the result in author list
		// then return result
		
		$author_url = false;
		
		if ($is_author_page)
		{
			echo "	Entering author logic.\n";
			// This is an author page, so search for rel=me instead
			// and loop through for one that references plus.google.com
			// as could be other handles here.
			
			$author_pattern = '/(<a\s+[^>]*)rel\s*=\s*[\'\"]me[^>]*>/';
			preg_match_all($author_pattern, $page_source, $matches);
			
			foreach ($matches[0] as $match)
			{
				echo "		Candidate URL = $match\n";
				
				//if (preg_match('/[plus|profile]\.google/', $match, $this_match))
				if (preg_match('/(([plus|profile]\.google\.com)|(google\.com\/profiles\/[0-9]+))/', $match, $this_match))
				{
					echo "			Match.\n";
					$author_url = $match;
					break 1;
				}
			}
			
		}else{
		
			echo "	Entering article logic.\n";
			// This isn't an author page, so just look for
			// the rel=author link.	
				
			$author_pattern = '/(<(a|g\:plus)\s+[^>]*)rel\s*=\s*([\'\"])?author[^>]*>/';
			preg_match($author_pattern, $page_source, $matches);
			
			if($matches)
			{
				$author_url = $matches[0];
			}
		}
		
		if($author_url) // We've found some authorship markup...
		{
			echo "	Searching URL $author_url for author ID.\n";
			
			$href_pattern = '/(.*href\s*=\s*[\'\"])([^\'\"]+)([^>]*>)/';
			$author_url = preg_replace($href_pattern, '$2', $author_url);
			
			// We have the authorship URL, now ascertain whether
			// it is a Google+ URL.
			
			// I have seen at least 5 valid profile URL formats (http or https both valid):
			// profiles.google.com/theney (this format 301s to G+ one below now.)
			// profiles.google.com/113217924531763968801
			// plus.google.com/113217924531763968801
			// plus.google.com/u/0/113217924531763968801
			// www.google.com/profiles/109412257237874861202 (damn you, Matt Cutts!)
			
			$google_pattern = '/(([plus|profile]\.google\.com)|(google\.com\/profiles\/[0-9]+))/';
			
			preg_match($google_pattern, $author_url, $matches);
			
			// if this is a Google+ URL
			if ($matches)
			{	
				echo "		This is a Google+ URL. Extracting ID... ";
				
				//$id_pattern = '/.*([plus|profile]\.google\.com\/)(u\/[0-9]\/)?([0-9]+)(\/)*\w*/';
				$id_pattern = '/.*([plus|profile|www]\.google\.com\/)(u\/[0-9]\/|profiles\/)?([0-9]+)(\/)*\w*/';
				
				preg_match($id_pattern, $author_url, $matches);
				
				// If we don't think it matches, then first we check if it's an old style
				// profile URL such as: profiles.google.com/theney
				// If so it lacks the ID number we need but it 301s so we can follow
				// the redirect and get the real URL.
				if (!$matches)
				{
					$old_profile_pattern = '/.*(profiles\.google\.com\/)([0-9a-zA-Z]+)(\/)*/';
					
					if (preg_match($old_profile_pattern, $author_url, $matches))
					{
						echo "Old style profile URL. Redirecting... ";
						$new_style_url = get_redirected_profile_url($author_url);
					
						if ($new_style_url)
						{
							$new_style_url = str_ireplace("http://", "", $new_style_url);
							$author_url = str_ireplace("https://", "", $new_style_url);
							
							// Now redo the match with the redirect URL.
							preg_match($id_pattern, $author_url, $matches);
						}
					}
				}
				
				// If we matched the ID 
				if ($matches)
				{
					$author_id = $matches[3];
					
					echo $author_id . "\n";
					
					if ($is_author_page)
						$author_ids_for_author_page[$page_url] = $author_id;
					
					return $author_id;
				}else{
					// Couldn't understand URL. :(
					
					if ($is_author_page)
						$author_ids_for_author_page[$page_url] = false;
						
					echo "			Failed to understand author URL.\n";
					
					return -1;
				}
			}else{
				echo "		This is an author bio URL. Visiting bio URL.\n";
				// if not a Google+ URL, lets check the author page
				
				// first be sure we are not already checking an author page
				if (!$is_author_page)
				{
					return get_page_author_id(rel2abs($author_url,$page_url), true);
				}else{
					// we're already checking an author page
					// and can't go deeper, no author found:
					return false;
				}
			}
			
		}else{
			// no authorship markup found:
			if ($is_author_page)
				$author_ids_for_author_page[$page_url] = false;
			return false;
		}
	    
	    return false;
	}
	
	
	// If a profile URL almost matches but has no numbers
	// it may be the old format where you were allowed a name
	// e.g https://profiles.google.com/theney
	// This function takes that and visits it to get the 301
	// URL for the new plus profile.
	function get_redirected_profile_url($profile_url)
	{
		$ch = curl_init();
		
		curl_setopt($ch, CURLOPT_URL, $profile_url);
		curl_setopt($ch, CURLOPT_TIMEOUT, 180);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_HEADER, true);
		curl_setopt($ch, CURLOPT_NOBODY, true);
		
		curl_exec($ch);
		$info = curl_getinfo($ch);
		curl_close($ch);
		
		if (!empty($info['redirect_url']))
		{
			return $info['redirect_url'];
		}
		
		return false;
	}
	
	// Slightly modified version of script I got here:
	// http://bsd-noobz.com/blog/php-script-for-converting-relative-to-absolute-url
	// Thanks to Nashruddin Amin.
	function rel2abs($rel, $base)
	{
	  if (parse_url($rel, PHP_URL_SCHEME) != '')
	    return $rel;
	  else if ($rel[0] == '#' || $rel[0] == '?')
	    return $base.$rel;
	
	
	  extract(parse_url($base));
	  
	  if ($scheme == "")
	  {
	  	extract(parse_url("http://" . $base));
	  }
	  
	  $abs = ($rel[0] == '/' ? '' : preg_replace('#/[^/]*$#', '', $path))."/$rel";
	  $re  = array('#(/\.?/)#', '#/(?!\.\.)[^/]+/\.\./#');
	
	  for ($n = 1; $n > 0; $abs = preg_replace($re, '/', $abs, -1, $n));
	  
	  return $scheme.'://'.$host.str_replace('../', '', $abs);
	}


?>